package com.smppw.analysis.domain.service.impl;

import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dao.FundInformationDao;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.NavService;
import com.smppw.common.pojo.IStrategy;
import com.smppw.common.pojo.dto.DateValue;
import com.smppw.common.pojo.dto.calc.IndicatorCalcPropertyDto;
import com.smppw.common.pojo.dto.indicator.CalcMultipleSecMultipleTimeRangeIndicatorReq;
import com.smppw.common.pojo.dto.indicator.DateIntervalDto;
import com.smppw.common.pojo.enums.*;
import com.smppw.core.IndicatorService;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class BaseIndicatorServiceV2Impl implements BaseIndicatorServiceV2 {
    private final NavService navService;
    private final FundInformationDao fundInformationDao;

    public BaseIndicatorServiceV2Impl(NavService navService, FundInformationDao fundInformationDao) {
        this.navService = navService;
        this.fundInformationDao = fundInformationDao;
    }

    @Override
    public Map<String, List<IndicatorCalcPropertyDto>> calcMultipleSecMultipleTimeRangeIndicator(CalcMultipleSecMultipleTimeRangeIndicatorReq req) {
        List<String> mainSecIdList = req.getMainSecIdList();
        Map<String, String> secBenchmarkIdMap = req.getSecBenchmarkIdMap();
        List<String> indexIdList = req.getIndexIdList();
        RaiseType raiseType = req.getRaiseType();
        IStrategy strategy = req.getStrategy();
        Visibility visibility = req.getVisibility();
        NavType navType = req.getNavType();
        String riskOfFreeId = req.getRiskOfFreeId();

        List<String> benchmarkIdList = secBenchmarkIdMap.values().stream().distinct().collect(Collectors.toList());
        CurveType curveType = CurveType.getCurveType(raiseType, strategy);
        if (StrUtil.isNotEmpty(riskOfFreeId)) {
            benchmarkIdList.add(riskOfFreeId);
        }
        Map<String, List<DateValue>> allNavMap = navService.getSecIdDateValueNavListMapFromRedisAndDB(mainSecIdList, benchmarkIdList, indexIdList,
                null, null, navType, visibility, curveType.getId(), strategy.getStrategyId(), new HashMap<>(), false);
        Map<String, Frequency> secFreqMap = this.fundInformationDao.getNavFrequency(mainSecIdList);
        return IndicatorService.getInstance().calcMultipleSecMultipleTimeRangeIndicator(req, allNavMap, secFreqMap);
    }

    @Override
    public Map<String, List<IndicatorCalcPropertyDto>> getMultipleSecTrend(List<String> mainSecIdList, Map<String, String> secBenchmarkIdMap,
                                                                           List<String> indexIdList, DateIntervalDto dateIntervalDto, Frequency frequency,
                                                                           BigDecimal fixedIncome, BigDecimal initValue, RaiseType raiseType, IStrategy strategy,
                                                                           Visibility visibility, NavType navType, List<TrendType> trendTypeV2List) {
        List<String> benchmarkIdList = secBenchmarkIdMap.values().stream().distinct().collect(Collectors.toList());
        CurveType curveType = CurveType.getCurveType(raiseType, strategy);
        Map<String, List<DateValue>> allNavMap = navService.getSecIdDateValueNavListMapFromRedisAndDB(mainSecIdList, benchmarkIdList, indexIdList,
                null, null, navType, visibility, curveType.getId(), strategy.getStrategyId(), new HashMap<>(), false);
        Map<String, Frequency> secFreqMap = this.fundInformationDao.getNavFrequency(mainSecIdList);
        return IndicatorService.getInstance().getMultipleSecTrend(mainSecIdList, secBenchmarkIdMap, indexIdList, dateIntervalDto, fixedIncome,
                initValue, frequency, trendTypeV2List, secFreqMap, allNavMap);
    }
}
